/// Result Enum dictating the exit condition for an optimization call
pub enum ExitCondition {
    /// Optimization has converged to user specified tolerance
    Converged,
    /// Optimization has exceeded user specified max iteration count
    MaxIter,
}

pub enum Direction {
    Minimize,
    Maximize,
}

impl Direction {
    pub fn factor(&self) -> f64 {
        match self {
            Direction::Minimize => -1.0,
            Direction::Maximize => 1.0,
        }
    }
}
/// Struct holding the results for a minimization call
pub struct OptimizationResult<T> {
    pub best_xs: T,
    pub best_fun_val: f64,
    pub exit_con: ExitCondition,
    pub iters: usize,
}
